# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from m3ter import M3ter, AsyncM3ter
from tests.utils import assert_matches_type
from m3ter._utils import parse_date, parse_datetime
from m3ter.pagination import SyncCursor, AsyncCursor
from m3ter.types.balances import (
    ChargeScheduleListResponse,
    ChargeScheduleCreateResponse,
    ChargeScheduleDeleteResponse,
    ChargeScheduleUpdateResponse,
    ChargeSchedulePreviewResponse,
    ChargeScheduleRetrieveResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestChargeSchedules:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: M3ter) -> None:
        charge_schedule = client.balances.charge_schedules.create(
            balance_id="balanceId",
            bill_frequency="DAILY",
            bill_frequency_interval=1,
            bill_in_advance=True,
            charge_description="x",
            code='S?oC"$]C] ]]]]]5]',
            currency="currency",
            name="x",
            service_period_end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            service_period_start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(ChargeScheduleCreateResponse, charge_schedule, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: M3ter) -> None:
        charge_schedule = client.balances.charge_schedules.create(
            balance_id="balanceId",
            bill_frequency="DAILY",
            bill_frequency_interval=1,
            bill_in_advance=True,
            charge_description="x",
            code='S?oC"$]C] ]]]]]5]',
            currency="currency",
            name="x",
            service_period_end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            service_period_start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            amount=0,
            bill_epoch=parse_date("2019-12-27"),
            custom_fields={"foo": "string"},
            unit_price=0,
            units=0,
            version=0,
        )
        assert_matches_type(ChargeScheduleCreateResponse, charge_schedule, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: M3ter) -> None:
        response = client.balances.charge_schedules.with_raw_response.create(
            balance_id="balanceId",
            bill_frequency="DAILY",
            bill_frequency_interval=1,
            bill_in_advance=True,
            charge_description="x",
            code='S?oC"$]C] ]]]]]5]',
            currency="currency",
            name="x",
            service_period_end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            service_period_start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        charge_schedule = response.parse()
        assert_matches_type(ChargeScheduleCreateResponse, charge_schedule, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: M3ter) -> None:
        with client.balances.charge_schedules.with_streaming_response.create(
            balance_id="balanceId",
            bill_frequency="DAILY",
            bill_frequency_interval=1,
            bill_in_advance=True,
            charge_description="x",
            code='S?oC"$]C] ]]]]]5]',
            currency="currency",
            name="x",
            service_period_end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            service_period_start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            charge_schedule = response.parse()
            assert_matches_type(ChargeScheduleCreateResponse, charge_schedule, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_create(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `balance_id` but received ''"):
            client.balances.charge_schedules.with_raw_response.create(
                balance_id="",
                bill_frequency="DAILY",
                bill_frequency_interval=1,
                bill_in_advance=True,
                charge_description="x",
                code='S?oC"$]C] ]]]]]5]',
                currency="currency",
                name="x",
                service_period_end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
                service_period_start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            )

    @parametrize
    def test_method_retrieve(self, client: M3ter) -> None:
        charge_schedule = client.balances.charge_schedules.retrieve(
            id="id",
            balance_id="balanceId",
        )
        assert_matches_type(ChargeScheduleRetrieveResponse, charge_schedule, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: M3ter) -> None:
        response = client.balances.charge_schedules.with_raw_response.retrieve(
            id="id",
            balance_id="balanceId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        charge_schedule = response.parse()
        assert_matches_type(ChargeScheduleRetrieveResponse, charge_schedule, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: M3ter) -> None:
        with client.balances.charge_schedules.with_streaming_response.retrieve(
            id="id",
            balance_id="balanceId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            charge_schedule = response.parse()
            assert_matches_type(ChargeScheduleRetrieveResponse, charge_schedule, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `balance_id` but received ''"):
            client.balances.charge_schedules.with_raw_response.retrieve(
                id="id",
                balance_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.balances.charge_schedules.with_raw_response.retrieve(
                id="",
                balance_id="balanceId",
            )

    @parametrize
    def test_method_update(self, client: M3ter) -> None:
        charge_schedule = client.balances.charge_schedules.update(
            id="id",
            balance_id="balanceId",
            bill_frequency="DAILY",
            bill_frequency_interval=1,
            bill_in_advance=True,
            charge_description="x",
            code='S?oC"$]C] ]]]]]5]',
            currency="currency",
            name="x",
            service_period_end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            service_period_start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(ChargeScheduleUpdateResponse, charge_schedule, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: M3ter) -> None:
        charge_schedule = client.balances.charge_schedules.update(
            id="id",
            balance_id="balanceId",
            bill_frequency="DAILY",
            bill_frequency_interval=1,
            bill_in_advance=True,
            charge_description="x",
            code='S?oC"$]C] ]]]]]5]',
            currency="currency",
            name="x",
            service_period_end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            service_period_start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            amount=0,
            bill_epoch=parse_date("2019-12-27"),
            custom_fields={"foo": "string"},
            unit_price=0,
            units=0,
            version=0,
        )
        assert_matches_type(ChargeScheduleUpdateResponse, charge_schedule, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: M3ter) -> None:
        response = client.balances.charge_schedules.with_raw_response.update(
            id="id",
            balance_id="balanceId",
            bill_frequency="DAILY",
            bill_frequency_interval=1,
            bill_in_advance=True,
            charge_description="x",
            code='S?oC"$]C] ]]]]]5]',
            currency="currency",
            name="x",
            service_period_end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            service_period_start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        charge_schedule = response.parse()
        assert_matches_type(ChargeScheduleUpdateResponse, charge_schedule, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: M3ter) -> None:
        with client.balances.charge_schedules.with_streaming_response.update(
            id="id",
            balance_id="balanceId",
            bill_frequency="DAILY",
            bill_frequency_interval=1,
            bill_in_advance=True,
            charge_description="x",
            code='S?oC"$]C] ]]]]]5]',
            currency="currency",
            name="x",
            service_period_end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            service_period_start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            charge_schedule = response.parse()
            assert_matches_type(ChargeScheduleUpdateResponse, charge_schedule, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `balance_id` but received ''"):
            client.balances.charge_schedules.with_raw_response.update(
                id="id",
                balance_id="",
                bill_frequency="DAILY",
                bill_frequency_interval=1,
                bill_in_advance=True,
                charge_description="x",
                code='S?oC"$]C] ]]]]]5]',
                currency="currency",
                name="x",
                service_period_end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
                service_period_start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.balances.charge_schedules.with_raw_response.update(
                id="",
                balance_id="balanceId",
                bill_frequency="DAILY",
                bill_frequency_interval=1,
                bill_in_advance=True,
                charge_description="x",
                code='S?oC"$]C] ]]]]]5]',
                currency="currency",
                name="x",
                service_period_end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
                service_period_start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            )

    @parametrize
    def test_method_list(self, client: M3ter) -> None:
        charge_schedule = client.balances.charge_schedules.list(
            balance_id="balanceId",
        )
        assert_matches_type(SyncCursor[ChargeScheduleListResponse], charge_schedule, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: M3ter) -> None:
        charge_schedule = client.balances.charge_schedules.list(
            balance_id="balanceId",
            ids=["string"],
            next_token="nextToken",
            page_size=1,
        )
        assert_matches_type(SyncCursor[ChargeScheduleListResponse], charge_schedule, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: M3ter) -> None:
        response = client.balances.charge_schedules.with_raw_response.list(
            balance_id="balanceId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        charge_schedule = response.parse()
        assert_matches_type(SyncCursor[ChargeScheduleListResponse], charge_schedule, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: M3ter) -> None:
        with client.balances.charge_schedules.with_streaming_response.list(
            balance_id="balanceId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            charge_schedule = response.parse()
            assert_matches_type(SyncCursor[ChargeScheduleListResponse], charge_schedule, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `balance_id` but received ''"):
            client.balances.charge_schedules.with_raw_response.list(
                balance_id="",
            )

    @parametrize
    def test_method_delete(self, client: M3ter) -> None:
        charge_schedule = client.balances.charge_schedules.delete(
            id="id",
            balance_id="balanceId",
        )
        assert_matches_type(ChargeScheduleDeleteResponse, charge_schedule, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: M3ter) -> None:
        response = client.balances.charge_schedules.with_raw_response.delete(
            id="id",
            balance_id="balanceId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        charge_schedule = response.parse()
        assert_matches_type(ChargeScheduleDeleteResponse, charge_schedule, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: M3ter) -> None:
        with client.balances.charge_schedules.with_streaming_response.delete(
            id="id",
            balance_id="balanceId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            charge_schedule = response.parse()
            assert_matches_type(ChargeScheduleDeleteResponse, charge_schedule, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `balance_id` but received ''"):
            client.balances.charge_schedules.with_raw_response.delete(
                id="id",
                balance_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.balances.charge_schedules.with_raw_response.delete(
                id="",
                balance_id="balanceId",
            )

    @parametrize
    def test_method_preview(self, client: M3ter) -> None:
        charge_schedule = client.balances.charge_schedules.preview(
            balance_id="balanceId",
            bill_frequency="DAILY",
            bill_frequency_interval=1,
            bill_in_advance=True,
            charge_description="x",
            code='S?oC"$]C] ]]]]]5]',
            currency="currency",
            name="x",
            service_period_end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            service_period_start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(ChargeSchedulePreviewResponse, charge_schedule, path=["response"])

    @parametrize
    def test_method_preview_with_all_params(self, client: M3ter) -> None:
        charge_schedule = client.balances.charge_schedules.preview(
            balance_id="balanceId",
            bill_frequency="DAILY",
            bill_frequency_interval=1,
            bill_in_advance=True,
            charge_description="x",
            code='S?oC"$]C] ]]]]]5]',
            currency="currency",
            name="x",
            service_period_end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            service_period_start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            next_token="nextToken",
            page_size=1,
            amount=0,
            bill_epoch=parse_date("2019-12-27"),
            custom_fields={"foo": "string"},
            unit_price=0,
            units=0,
            version=0,
        )
        assert_matches_type(ChargeSchedulePreviewResponse, charge_schedule, path=["response"])

    @parametrize
    def test_raw_response_preview(self, client: M3ter) -> None:
        response = client.balances.charge_schedules.with_raw_response.preview(
            balance_id="balanceId",
            bill_frequency="DAILY",
            bill_frequency_interval=1,
            bill_in_advance=True,
            charge_description="x",
            code='S?oC"$]C] ]]]]]5]',
            currency="currency",
            name="x",
            service_period_end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            service_period_start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        charge_schedule = response.parse()
        assert_matches_type(ChargeSchedulePreviewResponse, charge_schedule, path=["response"])

    @parametrize
    def test_streaming_response_preview(self, client: M3ter) -> None:
        with client.balances.charge_schedules.with_streaming_response.preview(
            balance_id="balanceId",
            bill_frequency="DAILY",
            bill_frequency_interval=1,
            bill_in_advance=True,
            charge_description="x",
            code='S?oC"$]C] ]]]]]5]',
            currency="currency",
            name="x",
            service_period_end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            service_period_start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            charge_schedule = response.parse()
            assert_matches_type(ChargeSchedulePreviewResponse, charge_schedule, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_preview(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `balance_id` but received ''"):
            client.balances.charge_schedules.with_raw_response.preview(
                balance_id="",
                bill_frequency="DAILY",
                bill_frequency_interval=1,
                bill_in_advance=True,
                charge_description="x",
                code='S?oC"$]C] ]]]]]5]',
                currency="currency",
                name="x",
                service_period_end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
                service_period_start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            )


class TestAsyncChargeSchedules:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncM3ter) -> None:
        charge_schedule = await async_client.balances.charge_schedules.create(
            balance_id="balanceId",
            bill_frequency="DAILY",
            bill_frequency_interval=1,
            bill_in_advance=True,
            charge_description="x",
            code='S?oC"$]C] ]]]]]5]',
            currency="currency",
            name="x",
            service_period_end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            service_period_start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(ChargeScheduleCreateResponse, charge_schedule, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncM3ter) -> None:
        charge_schedule = await async_client.balances.charge_schedules.create(
            balance_id="balanceId",
            bill_frequency="DAILY",
            bill_frequency_interval=1,
            bill_in_advance=True,
            charge_description="x",
            code='S?oC"$]C] ]]]]]5]',
            currency="currency",
            name="x",
            service_period_end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            service_period_start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            amount=0,
            bill_epoch=parse_date("2019-12-27"),
            custom_fields={"foo": "string"},
            unit_price=0,
            units=0,
            version=0,
        )
        assert_matches_type(ChargeScheduleCreateResponse, charge_schedule, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncM3ter) -> None:
        response = await async_client.balances.charge_schedules.with_raw_response.create(
            balance_id="balanceId",
            bill_frequency="DAILY",
            bill_frequency_interval=1,
            bill_in_advance=True,
            charge_description="x",
            code='S?oC"$]C] ]]]]]5]',
            currency="currency",
            name="x",
            service_period_end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            service_period_start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        charge_schedule = await response.parse()
        assert_matches_type(ChargeScheduleCreateResponse, charge_schedule, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncM3ter) -> None:
        async with async_client.balances.charge_schedules.with_streaming_response.create(
            balance_id="balanceId",
            bill_frequency="DAILY",
            bill_frequency_interval=1,
            bill_in_advance=True,
            charge_description="x",
            code='S?oC"$]C] ]]]]]5]',
            currency="currency",
            name="x",
            service_period_end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            service_period_start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            charge_schedule = await response.parse()
            assert_matches_type(ChargeScheduleCreateResponse, charge_schedule, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_create(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `balance_id` but received ''"):
            await async_client.balances.charge_schedules.with_raw_response.create(
                balance_id="",
                bill_frequency="DAILY",
                bill_frequency_interval=1,
                bill_in_advance=True,
                charge_description="x",
                code='S?oC"$]C] ]]]]]5]',
                currency="currency",
                name="x",
                service_period_end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
                service_period_start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            )

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncM3ter) -> None:
        charge_schedule = await async_client.balances.charge_schedules.retrieve(
            id="id",
            balance_id="balanceId",
        )
        assert_matches_type(ChargeScheduleRetrieveResponse, charge_schedule, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncM3ter) -> None:
        response = await async_client.balances.charge_schedules.with_raw_response.retrieve(
            id="id",
            balance_id="balanceId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        charge_schedule = await response.parse()
        assert_matches_type(ChargeScheduleRetrieveResponse, charge_schedule, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncM3ter) -> None:
        async with async_client.balances.charge_schedules.with_streaming_response.retrieve(
            id="id",
            balance_id="balanceId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            charge_schedule = await response.parse()
            assert_matches_type(ChargeScheduleRetrieveResponse, charge_schedule, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `balance_id` but received ''"):
            await async_client.balances.charge_schedules.with_raw_response.retrieve(
                id="id",
                balance_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.balances.charge_schedules.with_raw_response.retrieve(
                id="",
                balance_id="balanceId",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncM3ter) -> None:
        charge_schedule = await async_client.balances.charge_schedules.update(
            id="id",
            balance_id="balanceId",
            bill_frequency="DAILY",
            bill_frequency_interval=1,
            bill_in_advance=True,
            charge_description="x",
            code='S?oC"$]C] ]]]]]5]',
            currency="currency",
            name="x",
            service_period_end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            service_period_start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(ChargeScheduleUpdateResponse, charge_schedule, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncM3ter) -> None:
        charge_schedule = await async_client.balances.charge_schedules.update(
            id="id",
            balance_id="balanceId",
            bill_frequency="DAILY",
            bill_frequency_interval=1,
            bill_in_advance=True,
            charge_description="x",
            code='S?oC"$]C] ]]]]]5]',
            currency="currency",
            name="x",
            service_period_end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            service_period_start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            amount=0,
            bill_epoch=parse_date("2019-12-27"),
            custom_fields={"foo": "string"},
            unit_price=0,
            units=0,
            version=0,
        )
        assert_matches_type(ChargeScheduleUpdateResponse, charge_schedule, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncM3ter) -> None:
        response = await async_client.balances.charge_schedules.with_raw_response.update(
            id="id",
            balance_id="balanceId",
            bill_frequency="DAILY",
            bill_frequency_interval=1,
            bill_in_advance=True,
            charge_description="x",
            code='S?oC"$]C] ]]]]]5]',
            currency="currency",
            name="x",
            service_period_end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            service_period_start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        charge_schedule = await response.parse()
        assert_matches_type(ChargeScheduleUpdateResponse, charge_schedule, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncM3ter) -> None:
        async with async_client.balances.charge_schedules.with_streaming_response.update(
            id="id",
            balance_id="balanceId",
            bill_frequency="DAILY",
            bill_frequency_interval=1,
            bill_in_advance=True,
            charge_description="x",
            code='S?oC"$]C] ]]]]]5]',
            currency="currency",
            name="x",
            service_period_end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            service_period_start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            charge_schedule = await response.parse()
            assert_matches_type(ChargeScheduleUpdateResponse, charge_schedule, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `balance_id` but received ''"):
            await async_client.balances.charge_schedules.with_raw_response.update(
                id="id",
                balance_id="",
                bill_frequency="DAILY",
                bill_frequency_interval=1,
                bill_in_advance=True,
                charge_description="x",
                code='S?oC"$]C] ]]]]]5]',
                currency="currency",
                name="x",
                service_period_end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
                service_period_start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.balances.charge_schedules.with_raw_response.update(
                id="",
                balance_id="balanceId",
                bill_frequency="DAILY",
                bill_frequency_interval=1,
                bill_in_advance=True,
                charge_description="x",
                code='S?oC"$]C] ]]]]]5]',
                currency="currency",
                name="x",
                service_period_end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
                service_period_start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncM3ter) -> None:
        charge_schedule = await async_client.balances.charge_schedules.list(
            balance_id="balanceId",
        )
        assert_matches_type(AsyncCursor[ChargeScheduleListResponse], charge_schedule, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncM3ter) -> None:
        charge_schedule = await async_client.balances.charge_schedules.list(
            balance_id="balanceId",
            ids=["string"],
            next_token="nextToken",
            page_size=1,
        )
        assert_matches_type(AsyncCursor[ChargeScheduleListResponse], charge_schedule, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncM3ter) -> None:
        response = await async_client.balances.charge_schedules.with_raw_response.list(
            balance_id="balanceId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        charge_schedule = await response.parse()
        assert_matches_type(AsyncCursor[ChargeScheduleListResponse], charge_schedule, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncM3ter) -> None:
        async with async_client.balances.charge_schedules.with_streaming_response.list(
            balance_id="balanceId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            charge_schedule = await response.parse()
            assert_matches_type(AsyncCursor[ChargeScheduleListResponse], charge_schedule, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `balance_id` but received ''"):
            await async_client.balances.charge_schedules.with_raw_response.list(
                balance_id="",
            )

    @parametrize
    async def test_method_delete(self, async_client: AsyncM3ter) -> None:
        charge_schedule = await async_client.balances.charge_schedules.delete(
            id="id",
            balance_id="balanceId",
        )
        assert_matches_type(ChargeScheduleDeleteResponse, charge_schedule, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncM3ter) -> None:
        response = await async_client.balances.charge_schedules.with_raw_response.delete(
            id="id",
            balance_id="balanceId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        charge_schedule = await response.parse()
        assert_matches_type(ChargeScheduleDeleteResponse, charge_schedule, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncM3ter) -> None:
        async with async_client.balances.charge_schedules.with_streaming_response.delete(
            id="id",
            balance_id="balanceId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            charge_schedule = await response.parse()
            assert_matches_type(ChargeScheduleDeleteResponse, charge_schedule, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `balance_id` but received ''"):
            await async_client.balances.charge_schedules.with_raw_response.delete(
                id="id",
                balance_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.balances.charge_schedules.with_raw_response.delete(
                id="",
                balance_id="balanceId",
            )

    @parametrize
    async def test_method_preview(self, async_client: AsyncM3ter) -> None:
        charge_schedule = await async_client.balances.charge_schedules.preview(
            balance_id="balanceId",
            bill_frequency="DAILY",
            bill_frequency_interval=1,
            bill_in_advance=True,
            charge_description="x",
            code='S?oC"$]C] ]]]]]5]',
            currency="currency",
            name="x",
            service_period_end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            service_period_start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(ChargeSchedulePreviewResponse, charge_schedule, path=["response"])

    @parametrize
    async def test_method_preview_with_all_params(self, async_client: AsyncM3ter) -> None:
        charge_schedule = await async_client.balances.charge_schedules.preview(
            balance_id="balanceId",
            bill_frequency="DAILY",
            bill_frequency_interval=1,
            bill_in_advance=True,
            charge_description="x",
            code='S?oC"$]C] ]]]]]5]',
            currency="currency",
            name="x",
            service_period_end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            service_period_start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            next_token="nextToken",
            page_size=1,
            amount=0,
            bill_epoch=parse_date("2019-12-27"),
            custom_fields={"foo": "string"},
            unit_price=0,
            units=0,
            version=0,
        )
        assert_matches_type(ChargeSchedulePreviewResponse, charge_schedule, path=["response"])

    @parametrize
    async def test_raw_response_preview(self, async_client: AsyncM3ter) -> None:
        response = await async_client.balances.charge_schedules.with_raw_response.preview(
            balance_id="balanceId",
            bill_frequency="DAILY",
            bill_frequency_interval=1,
            bill_in_advance=True,
            charge_description="x",
            code='S?oC"$]C] ]]]]]5]',
            currency="currency",
            name="x",
            service_period_end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            service_period_start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        charge_schedule = await response.parse()
        assert_matches_type(ChargeSchedulePreviewResponse, charge_schedule, path=["response"])

    @parametrize
    async def test_streaming_response_preview(self, async_client: AsyncM3ter) -> None:
        async with async_client.balances.charge_schedules.with_streaming_response.preview(
            balance_id="balanceId",
            bill_frequency="DAILY",
            bill_frequency_interval=1,
            bill_in_advance=True,
            charge_description="x",
            code='S?oC"$]C] ]]]]]5]',
            currency="currency",
            name="x",
            service_period_end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            service_period_start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            charge_schedule = await response.parse()
            assert_matches_type(ChargeSchedulePreviewResponse, charge_schedule, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_preview(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `balance_id` but received ''"):
            await async_client.balances.charge_schedules.with_raw_response.preview(
                balance_id="",
                bill_frequency="DAILY",
                bill_frequency_interval=1,
                bill_in_advance=True,
                charge_description="x",
                code='S?oC"$]C] ]]]]]5]',
                currency="currency",
                name="x",
                service_period_end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
                service_period_start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["StatementDefinitionResponse", "Dimension", "Measure"]


class Dimension(BaseModel):
    """A Dimension belonging to a Meter."""

    filter: List[str]
    """The value of a Dimension to use as a filter.

    Use "\\**" as a wildcard to filter on all Dimension values.
    """

    name: str
    """The name of the Dimension to target in the Meter."""

    attributes: Optional[List[str]] = None
    """The Dimension attribute to target."""

    meter_id: Optional[str] = FieldInfo(alias="meterId", default=None)
    """The unique identifier (UUID) of the Meter containing this Dimension."""


class Measure(BaseModel):
    aggregations: Optional[List[Literal["SUM", "MIN", "MAX", "COUNT", "LATEST", "MEAN", "UNIQUE", "CUSTOM_SQL"]]] = None
    """A list of Aggregations to apply to the Measure."""

    meter_id: Optional[str] = FieldInfo(alias="meterId", default=None)
    """The unique identifier (UUID) of the Meter containing this Measure."""

    name: Optional[str] = None
    """The name of a Measure data field \\**(or blank to indicate a wildcard, i.e.

    all fields)\\**. Default value is blank.
    """


class StatementDefinitionResponse(BaseModel):
    id: str
    """The UUID of the entity."""

    aggregation_frequency: Optional[Literal["DAY", "WEEK", "MONTH", "QUARTER", "YEAR", "WHOLE_PERIOD"]] = FieldInfo(
        alias="aggregationFrequency", default=None
    )
    """This specifies how often the Statement should aggregate data."""

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """The unique identifier (UUID) of the user who created this StatementDefinition."""

    dimensions: Optional[List[Dimension]] = None
    """
    An array of objects, each representing a Dimension data field from a Meter _(for
    Meters that have Dimensions setup)_.
    """

    dt_created: Optional[datetime] = FieldInfo(alias="dtCreated", default=None)
    """
    The date and time _(in ISO-8601 format)_ when the StatementDefinition was
    created.
    """

    dt_last_modified: Optional[datetime] = FieldInfo(alias="dtLastModified", default=None)
    """
    The date and time _(in ISO-8601 format)_ when the StatementDefinition was last
    modified.
    """

    generate_slim_statements: Optional[bool] = FieldInfo(alias="generateSlimStatements", default=None)

    include_price_per_unit: Optional[bool] = FieldInfo(alias="includePricePerUnit", default=None)
    """A Boolean indicating whether to include the price per unit in the Statement.

    - TRUE - includes the price per unit.
    - FALSE - excludes the price per unit.
    """

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """
    The unique identifier (UUID) of the user who last modified this
    StatementDefinition.
    """

    measures: Optional[List[Measure]] = None
    """An array of objects, each representing a Measure data field from a Meter."""

    name: Optional[str] = None
    """Descriptive name for the StatementDefinition providing context and information."""

    version: Optional[int] = None
    """The version number:

    - **Create:** On initial Create to insert a new entity, the version is set at 1
      in the response.
    - **Update:** On successful Update, the version is incremented by 1 in the
      response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["DataExplorerGroup"]


class DataExplorerGroup(BaseModel):
    """Group by a field"""

    group_type: Optional[Literal["ACCOUNT", "DIMENSION", "TIME"]] = FieldInfo(alias="groupType", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from m3ter import M3ter, AsyncM3ter
from m3ter.types import (
    AggregationResponse,
    CompoundAggregationResponse,
)
from tests.utils import assert_matches_type
from m3ter.pagination import SyncCursor, AsyncCursor

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestCompoundAggregations:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: M3ter) -> None:
        compound_aggregation = client.compound_aggregations.create(
            calculation="x",
            name="x",
            quantity_per_unit=1,
            rounding="UP",
            unit="x",
        )
        assert_matches_type(AggregationResponse, compound_aggregation, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: M3ter) -> None:
        compound_aggregation = client.compound_aggregations.create(
            calculation="x",
            name="x",
            quantity_per_unit=1,
            rounding="UP",
            unit="x",
            accounting_product_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            code="example_code",
            custom_fields={"foo": "string"},
            evaluate_null_aggregations=True,
            product_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            version=0,
        )
        assert_matches_type(AggregationResponse, compound_aggregation, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: M3ter) -> None:
        response = client.compound_aggregations.with_raw_response.create(
            calculation="x",
            name="x",
            quantity_per_unit=1,
            rounding="UP",
            unit="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        compound_aggregation = response.parse()
        assert_matches_type(AggregationResponse, compound_aggregation, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: M3ter) -> None:
        with client.compound_aggregations.with_streaming_response.create(
            calculation="x",
            name="x",
            quantity_per_unit=1,
            rounding="UP",
            unit="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            compound_aggregation = response.parse()
            assert_matches_type(AggregationResponse, compound_aggregation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: M3ter) -> None:
        compound_aggregation = client.compound_aggregations.retrieve(
            id="id",
        )
        assert_matches_type(CompoundAggregationResponse, compound_aggregation, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: M3ter) -> None:
        response = client.compound_aggregations.with_raw_response.retrieve(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        compound_aggregation = response.parse()
        assert_matches_type(CompoundAggregationResponse, compound_aggregation, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: M3ter) -> None:
        with client.compound_aggregations.with_streaming_response.retrieve(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            compound_aggregation = response.parse()
            assert_matches_type(CompoundAggregationResponse, compound_aggregation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.compound_aggregations.with_raw_response.retrieve(
                id="",
            )

    @parametrize
    def test_method_update(self, client: M3ter) -> None:
        compound_aggregation = client.compound_aggregations.update(
            id="id",
            calculation="x",
            name="x",
            quantity_per_unit=1,
            rounding="UP",
            unit="x",
        )
        assert_matches_type(AggregationResponse, compound_aggregation, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: M3ter) -> None:
        compound_aggregation = client.compound_aggregations.update(
            id="id",
            calculation="x",
            name="x",
            quantity_per_unit=1,
            rounding="UP",
            unit="x",
            accounting_product_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            code="example_code",
            custom_fields={"foo": "string"},
            evaluate_null_aggregations=True,
            product_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            version=0,
        )
        assert_matches_type(AggregationResponse, compound_aggregation, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: M3ter) -> None:
        response = client.compound_aggregations.with_raw_response.update(
            id="id",
            calculation="x",
            name="x",
            quantity_per_unit=1,
            rounding="UP",
            unit="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        compound_aggregation = response.parse()
        assert_matches_type(AggregationResponse, compound_aggregation, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: M3ter) -> None:
        with client.compound_aggregations.with_streaming_response.update(
            id="id",
            calculation="x",
            name="x",
            quantity_per_unit=1,
            rounding="UP",
            unit="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            compound_aggregation = response.parse()
            assert_matches_type(AggregationResponse, compound_aggregation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.compound_aggregations.with_raw_response.update(
                id="",
                calculation="x",
                name="x",
                quantity_per_unit=1,
                rounding="UP",
                unit="x",
            )

    @parametrize
    def test_method_list(self, client: M3ter) -> None:
        compound_aggregation = client.compound_aggregations.list()
        assert_matches_type(SyncCursor[CompoundAggregationResponse], compound_aggregation, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: M3ter) -> None:
        compound_aggregation = client.compound_aggregations.list(
            codes=["string"],
            ids=["string"],
            next_token="nextToken",
            page_size=1,
            product_id=["string"],
        )
        assert_matches_type(SyncCursor[CompoundAggregationResponse], compound_aggregation, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: M3ter) -> None:
        response = client.compound_aggregations.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        compound_aggregation = response.parse()
        assert_matches_type(SyncCursor[CompoundAggregationResponse], compound_aggregation, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: M3ter) -> None:
        with client.compound_aggregations.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            compound_aggregation = response.parse()
            assert_matches_type(SyncCursor[CompoundAggregationResponse], compound_aggregation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: M3ter) -> None:
        compound_aggregation = client.compound_aggregations.delete(
            id="id",
        )
        assert_matches_type(CompoundAggregationResponse, compound_aggregation, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: M3ter) -> None:
        response = client.compound_aggregations.with_raw_response.delete(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        compound_aggregation = response.parse()
        assert_matches_type(CompoundAggregationResponse, compound_aggregation, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: M3ter) -> None:
        with client.compound_aggregations.with_streaming_response.delete(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            compound_aggregation = response.parse()
            assert_matches_type(CompoundAggregationResponse, compound_aggregation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.compound_aggregations.with_raw_response.delete(
                id="",
            )


class TestAsyncCompoundAggregations:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncM3ter) -> None:
        compound_aggregation = await async_client.compound_aggregations.create(
            calculation="x",
            name="x",
            quantity_per_unit=1,
            rounding="UP",
            unit="x",
        )
        assert_matches_type(AggregationResponse, compound_aggregation, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncM3ter) -> None:
        compound_aggregation = await async_client.compound_aggregations.create(
            calculation="x",
            name="x",
            quantity_per_unit=1,
            rounding="UP",
            unit="x",
            accounting_product_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            code="example_code",
            custom_fields={"foo": "string"},
            evaluate_null_aggregations=True,
            product_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            version=0,
        )
        assert_matches_type(AggregationResponse, compound_aggregation, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncM3ter) -> None:
        response = await async_client.compound_aggregations.with_raw_response.create(
            calculation="x",
            name="x",
            quantity_per_unit=1,
            rounding="UP",
            unit="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        compound_aggregation = await response.parse()
        assert_matches_type(AggregationResponse, compound_aggregation, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncM3ter) -> None:
        async with async_client.compound_aggregations.with_streaming_response.create(
            calculation="x",
            name="x",
            quantity_per_unit=1,
            rounding="UP",
            unit="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            compound_aggregation = await response.parse()
            assert_matches_type(AggregationResponse, compound_aggregation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncM3ter) -> None:
        compound_aggregation = await async_client.compound_aggregations.retrieve(
            id="id",
        )
        assert_matches_type(CompoundAggregationResponse, compound_aggregation, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncM3ter) -> None:
        response = await async_client.compound_aggregations.with_raw_response.retrieve(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        compound_aggregation = await response.parse()
        assert_matches_type(CompoundAggregationResponse, compound_aggregation, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncM3ter) -> None:
        async with async_client.compound_aggregations.with_streaming_response.retrieve(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            compound_aggregation = await response.parse()
            assert_matches_type(CompoundAggregationResponse, compound_aggregation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.compound_aggregations.with_raw_response.retrieve(
                id="",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncM3ter) -> None:
        compound_aggregation = await async_client.compound_aggregations.update(
            id="id",
            calculation="x",
            name="x",
            quantity_per_unit=1,
            rounding="UP",
            unit="x",
        )
        assert_matches_type(AggregationResponse, compound_aggregation, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncM3ter) -> None:
        compound_aggregation = await async_client.compound_aggregations.update(
            id="id",
            calculation="x",
            name="x",
            quantity_per_unit=1,
            rounding="UP",
            unit="x",
            accounting_product_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            code="example_code",
            custom_fields={"foo": "string"},
            evaluate_null_aggregations=True,
            product_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            version=0,
        )
        assert_matches_type(AggregationResponse, compound_aggregation, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncM3ter) -> None:
        response = await async_client.compound_aggregations.with_raw_response.update(
            id="id",
            calculation="x",
            name="x",
            quantity_per_unit=1,
            rounding="UP",
            unit="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        compound_aggregation = await response.parse()
        assert_matches_type(AggregationResponse, compound_aggregation, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncM3ter) -> None:
        async with async_client.compound_aggregations.with_streaming_response.update(
            id="id",
            calculation="x",
            name="x",
            quantity_per_unit=1,
            rounding="UP",
            unit="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            compound_aggregation = await response.parse()
            assert_matches_type(AggregationResponse, compound_aggregation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.compound_aggregations.with_raw_response.update(
                id="",
                calculation="x",
                name="x",
                quantity_per_unit=1,
                rounding="UP",
                unit="x",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncM3ter) -> None:
        compound_aggregation = await async_client.compound_aggregations.list()
        assert_matches_type(AsyncCursor[CompoundAggregationResponse], compound_aggregation, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncM3ter) -> None:
        compound_aggregation = await async_client.compound_aggregations.list(
            codes=["string"],
            ids=["string"],
            next_token="nextToken",
            page_size=1,
            product_id=["string"],
        )
        assert_matches_type(AsyncCursor[CompoundAggregationResponse], compound_aggregation, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncM3ter) -> None:
        response = await async_client.compound_aggregations.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        compound_aggregation = await response.parse()
        assert_matches_type(AsyncCursor[CompoundAggregationResponse], compound_aggregation, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncM3ter) -> None:
        async with async_client.compound_aggregations.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            compound_aggregation = await response.parse()
            assert_matches_type(AsyncCursor[CompoundAggregationResponse], compound_aggregation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncM3ter) -> None:
        compound_aggregation = await async_client.compound_aggregations.delete(
            id="id",
        )
        assert_matches_type(CompoundAggregationResponse, compound_aggregation, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncM3ter) -> None:
        response = await async_client.compound_aggregations.with_raw_response.delete(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        compound_aggregation = await response.parse()
        assert_matches_type(CompoundAggregationResponse, compound_aggregation, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncM3ter) -> None:
        async with async_client.compound_aggregations.with_streaming_response.delete(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            compound_aggregation = await response.parse()
            assert_matches_type(CompoundAggregationResponse, compound_aggregation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.compound_aggregations.with_raw_response.delete(
                id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import transaction_type_list_params, transaction_type_create_params, transaction_type_update_params
from .._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursor, AsyncCursor
from .._base_client import AsyncPaginator, make_request_options
from ..types.transaction_type_response import TransactionTypeResponse

__all__ = ["TransactionTypesResource", "AsyncTransactionTypesResource"]


class TransactionTypesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TransactionTypesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return TransactionTypesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TransactionTypesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return TransactionTypesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        org_id: str | None = None,
        name: str,
        archived: bool | Omit = omit,
        code: str | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TransactionTypeResponse:
        """Create a new TransactionType for the specified Organization.

        Details of the new
        TransactionType should be included in the request body.

        Args:
          name: The name of the entity.

          archived: A Boolean TRUE / FALSE flag indicating whether the entity is archived. An entity
              can be archived if it is obsolete.

              - TRUE - the entity is in the archived state.
              - FALSE - the entity is not in the archived state.

          code: The short code for the entity.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._post(
            f"/organizations/{org_id}/picklists/transactiontypes",
            body=maybe_transform(
                {
                    "name": name,
                    "archived": archived,
                    "code": code,
                    "version": version,
                },
                transaction_type_create_params.TransactionTypeCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TransactionTypeResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TransactionTypeResponse:
        """
        Retrieves the TransactionType with the given UUID from the specified
        Organization.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/picklists/transactiontypes/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TransactionTypeResponse,
        )

    def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        name: str,
        archived: bool | Omit = omit,
        code: str | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TransactionTypeResponse:
        """
        Updates the TransactionType with the specified UUID for the specified
        Organization. Update details for the TransactionType should be included in the
        request body.

        Args:
          name: The name of the entity.

          archived: A Boolean TRUE / FALSE flag indicating whether the entity is archived. An entity
              can be archived if it is obsolete.

              - TRUE - the entity is in the archived state.
              - FALSE - the entity is not in the archived state.

          code: The short code for the entity.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/organizations/{org_id}/picklists/transactiontypes/{id}",
            body=maybe_transform(
                {
                    "name": name,
                    "archived": archived,
                    "code": code,
                    "version": version,
                },
                transaction_type_update_params.TransactionTypeUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TransactionTypeResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        archived: bool | Omit = omit,
        codes: SequenceNotStr[str] | Omit = omit,
        ids: SequenceNotStr[str] | Omit = omit,
        next_token: str | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursor[TransactionTypeResponse]:
        """Retrieves a list of TransactionType entities for the specified Organization.

        The
        list can be paginated for easier management, and supports filtering by various
        parameters.

        Args:
          archived: Filter with this Boolean flag whether to include TransactionTypes that are
              archived.

              - TRUE - include archived TransactionTypes in the list.
              - FALSE - exclude archived TransactionTypes.

          codes: A list of TransactionType short codes to retrieve.

          ids: A list of TransactionType unique identifiers (UUIDs) to retrieve.

          next_token: The `nextToken` for multi-page retrievals. It is used to fetch the next page of
              TransactionTypes in a paginated list.

          page_size: Specifies the maximum number of TransactionTypes to retrieve per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/picklists/transactiontypes",
            page=SyncCursor[TransactionTypeResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "archived": archived,
                        "codes": codes,
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    transaction_type_list_params.TransactionTypeListParams,
                ),
            ),
            model=TransactionTypeResponse,
        )

    def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TransactionTypeResponse:
        """
        Deletes the TransactionType with the given UUID from the specified Organization.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/organizations/{org_id}/picklists/transactiontypes/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TransactionTypeResponse,
        )


class AsyncTransactionTypesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTransactionTypesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTransactionTypesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTransactionTypesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncTransactionTypesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        org_id: str | None = None,
        name: str,
        archived: bool | Omit = omit,
        code: str | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TransactionTypeResponse:
        """Create a new TransactionType for the specified Organization.

        Details of the new
        TransactionType should be included in the request body.

        Args:
          name: The name of the entity.

          archived: A Boolean TRUE / FALSE flag indicating whether the entity is archived. An entity
              can be archived if it is obsolete.

              - TRUE - the entity is in the archived state.
              - FALSE - the entity is not in the archived state.

          code: The short code for the entity.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._post(
            f"/organizations/{org_id}/picklists/transactiontypes",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "archived": archived,
                    "code": code,
                    "version": version,
                },
                transaction_type_create_params.TransactionTypeCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TransactionTypeResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TransactionTypeResponse:
        """
        Retrieves the TransactionType with the given UUID from the specified
        Organization.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/picklists/transactiontypes/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TransactionTypeResponse,
        )

    async def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        name: str,
        archived: bool | Omit = omit,
        code: str | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TransactionTypeResponse:
        """
        Updates the TransactionType with the specified UUID for the specified
        Organization. Update details for the TransactionType should be included in the
        request body.

        Args:
          name: The name of the entity.

          archived: A Boolean TRUE / FALSE flag indicating whether the entity is archived. An entity
              can be archived if it is obsolete.

              - TRUE - the entity is in the archived state.
              - FALSE - the entity is not in the archived state.

          code: The short code for the entity.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/organizations/{org_id}/picklists/transactiontypes/{id}",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "archived": archived,
                    "code": code,
                    "version": version,
                },
                transaction_type_update_params.TransactionTypeUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TransactionTypeResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        archived: bool | Omit = omit,
        codes: SequenceNotStr[str] | Omit = omit,
        ids: SequenceNotStr[str] | Omit = omit,
        next_token: str | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[TransactionTypeResponse, AsyncCursor[TransactionTypeResponse]]:
        """Retrieves a list of TransactionType entities for the specified Organization.

        The
        list can be paginated for easier management, and supports filtering by various
        parameters.

        Args:
          archived: Filter with this Boolean flag whether to include TransactionTypes that are
              archived.

              - TRUE - include archived TransactionTypes in the list.
              - FALSE - exclude archived TransactionTypes.

          codes: A list of TransactionType short codes to retrieve.

          ids: A list of TransactionType unique identifiers (UUIDs) to retrieve.

          next_token: The `nextToken` for multi-page retrievals. It is used to fetch the next page of
              TransactionTypes in a paginated list.

          page_size: Specifies the maximum number of TransactionTypes to retrieve per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/picklists/transactiontypes",
            page=AsyncCursor[TransactionTypeResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "archived": archived,
                        "codes": codes,
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    transaction_type_list_params.TransactionTypeListParams,
                ),
            ),
            model=TransactionTypeResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TransactionTypeResponse:
        """
        Deletes the TransactionType with the given UUID from the specified Organization.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/organizations/{org_id}/picklists/transactiontypes/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TransactionTypeResponse,
        )


class TransactionTypesResourceWithRawResponse:
    def __init__(self, transaction_types: TransactionTypesResource) -> None:
        self._transaction_types = transaction_types

        self.create = to_raw_response_wrapper(
            transaction_types.create,
        )
        self.retrieve = to_raw_response_wrapper(
            transaction_types.retrieve,
        )
        self.update = to_raw_response_wrapper(
            transaction_types.update,
        )
        self.list = to_raw_response_wrapper(
            transaction_types.list,
        )
        self.delete = to_raw_response_wrapper(
            transaction_types.delete,
        )


class AsyncTransactionTypesResourceWithRawResponse:
    def __init__(self, transaction_types: AsyncTransactionTypesResource) -> None:
        self._transaction_types = transaction_types

        self.create = async_to_raw_response_wrapper(
            transaction_types.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            transaction_types.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            transaction_types.update,
        )
        self.list = async_to_raw_response_wrapper(
            transaction_types.list,
        )
        self.delete = async_to_raw_response_wrapper(
            transaction_types.delete,
        )


class TransactionTypesResourceWithStreamingResponse:
    def __init__(self, transaction_types: TransactionTypesResource) -> None:
        self._transaction_types = transaction_types

        self.create = to_streamed_response_wrapper(
            transaction_types.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            transaction_types.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            transaction_types.update,
        )
        self.list = to_streamed_response_wrapper(
            transaction_types.list,
        )
        self.delete = to_streamed_response_wrapper(
            transaction_types.delete,
        )


class AsyncTransactionTypesResourceWithStreamingResponse:
    def __init__(self, transaction_types: AsyncTransactionTypesResource) -> None:
        self._transaction_types = transaction_types

        self.create = async_to_streamed_response_wrapper(
            transaction_types.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            transaction_types.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            transaction_types.update,
        )
        self.list = async_to_streamed_response_wrapper(
            transaction_types.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            transaction_types.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .commitment_response import CommitmentResponse

__all__ = ["CommitmentSearchResponse"]


class CommitmentSearchResponse(BaseModel):
    data: Optional[List[CommitmentResponse]] = None

    next_token: Optional[str] = FieldInfo(alias="nextToken", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .lookup_table_revision_data import (
    LookupTableRevisionDataResource,
    AsyncLookupTableRevisionDataResource,
    LookupTableRevisionDataResourceWithRawResponse,
    AsyncLookupTableRevisionDataResourceWithRawResponse,
    LookupTableRevisionDataResourceWithStreamingResponse,
    AsyncLookupTableRevisionDataResourceWithStreamingResponse,
)
from .lookup_table_revision_data_jobs import (
    LookupTableRevisionDataJobsResource,
    AsyncLookupTableRevisionDataJobsResource,
    LookupTableRevisionDataJobsResourceWithRawResponse,
    AsyncLookupTableRevisionDataJobsResourceWithRawResponse,
    LookupTableRevisionDataJobsResourceWithStreamingResponse,
    AsyncLookupTableRevisionDataJobsResourceWithStreamingResponse,
)

__all__ = [
    "LookupTableRevisionDataJobsResource",
    "AsyncLookupTableRevisionDataJobsResource",
    "LookupTableRevisionDataJobsResourceWithRawResponse",
    "AsyncLookupTableRevisionDataJobsResourceWithRawResponse",
    "LookupTableRevisionDataJobsResourceWithStreamingResponse",
    "AsyncLookupTableRevisionDataJobsResourceWithStreamingResponse",
    "LookupTableRevisionDataResource",
    "AsyncLookupTableRevisionDataResource",
    "LookupTableRevisionDataResourceWithRawResponse",
    "AsyncLookupTableRevisionDataResourceWithRawResponse",
    "LookupTableRevisionDataResourceWithStreamingResponse",
    "AsyncLookupTableRevisionDataResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from m3ter import M3ter, AsyncM3ter
from m3ter.types import AdHocResponse
from tests.utils import assert_matches_type
from m3ter._utils import parse_datetime

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestDataExports:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create_adhoc_overload_1(self, client: M3ter) -> None:
        data_export = client.data_exports.create_adhoc(
            operational_data_types=["BILLS"],
            source_type="OPERATIONAL",
        )
        assert_matches_type(AdHocResponse, data_export, path=["response"])

    @parametrize
    def test_method_create_adhoc_with_all_params_overload_1(self, client: M3ter) -> None:
        data_export = client.data_exports.create_adhoc(
            operational_data_types=["BILLS"],
            source_type="OPERATIONAL",
            version=0,
        )
        assert_matches_type(AdHocResponse, data_export, path=["response"])

    @parametrize
    def test_raw_response_create_adhoc_overload_1(self, client: M3ter) -> None:
        response = client.data_exports.with_raw_response.create_adhoc(
            operational_data_types=["BILLS"],
            source_type="OPERATIONAL",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        data_export = response.parse()
        assert_matches_type(AdHocResponse, data_export, path=["response"])

    @parametrize
    def test_streaming_response_create_adhoc_overload_1(self, client: M3ter) -> None:
        with client.data_exports.with_streaming_response.create_adhoc(
            operational_data_types=["BILLS"],
            source_type="OPERATIONAL",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            data_export = response.parse()
            assert_matches_type(AdHocResponse, data_export, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_create_adhoc_overload_2(self, client: M3ter) -> None:
        data_export = client.data_exports.create_adhoc(
            source_type="USAGE",
        )
        assert_matches_type(AdHocResponse, data_export, path=["response"])

    @parametrize
    def test_method_create_adhoc_with_all_params_overload_2(self, client: M3ter) -> None:
        data_export = client.data_exports.create_adhoc(
            source_type="USAGE",
            account_ids=["string"],
            aggregations=[
                {
                    "field_code": "x",
                    "field_type": "DIMENSION",
                    "function": "SUM",
                    "meter_id": "x",
                }
            ],
            dimension_filters=[
                {
                    "field_code": "x",
                    "meter_id": "x",
                    "values": ["string"],
                }
            ],
            end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            groups=[{"group_type": "ACCOUNT"}],
            meter_ids=["string"],
            version=0,
        )
        assert_matches_type(AdHocResponse, data_export, path=["response"])

    @parametrize
    def test_raw_response_create_adhoc_overload_2(self, client: M3ter) -> None:
        response = client.data_exports.with_raw_response.create_adhoc(
            source_type="USAGE",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        data_export = response.parse()
        assert_matches_type(AdHocResponse, data_export, path=["response"])

    @parametrize
    def test_streaming_response_create_adhoc_overload_2(self, client: M3ter) -> None:
        with client.data_exports.with_streaming_response.create_adhoc(
            source_type="USAGE",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            data_export = response.parse()
            assert_matches_type(AdHocResponse, data_export, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncDataExports:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create_adhoc_overload_1(self, async_client: AsyncM3ter) -> None:
        data_export = await async_client.data_exports.create_adhoc(
            operational_data_types=["BILLS"],
            source_type="OPERATIONAL",
        )
        assert_matches_type(AdHocResponse, data_export, path=["response"])

    @parametrize
    async def test_method_create_adhoc_with_all_params_overload_1(self, async_client: AsyncM3ter) -> None:
        data_export = await async_client.data_exports.create_adhoc(
            operational_data_types=["BILLS"],
            source_type="OPERATIONAL",
            version=0,
        )
        assert_matches_type(AdHocResponse, data_export, path=["response"])

    @parametrize
    async def test_raw_response_create_adhoc_overload_1(self, async_client: AsyncM3ter) -> None:
        response = await async_client.data_exports.with_raw_response.create_adhoc(
            operational_data_types=["BILLS"],
            source_type="OPERATIONAL",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        data_export = await response.parse()
        assert_matches_type(AdHocResponse, data_export, path=["response"])

    @parametrize
    async def test_streaming_response_create_adhoc_overload_1(self, async_client: AsyncM3ter) -> None:
        async with async_client.data_exports.with_streaming_response.create_adhoc(
            operational_data_types=["BILLS"],
            source_type="OPERATIONAL",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            data_export = await response.parse()
            assert_matches_type(AdHocResponse, data_export, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_create_adhoc_overload_2(self, async_client: AsyncM3ter) -> None:
        data_export = await async_client.data_exports.create_adhoc(
            source_type="USAGE",
        )
        assert_matches_type(AdHocResponse, data_export, path=["response"])

    @parametrize
    async def test_method_create_adhoc_with_all_params_overload_2(self, async_client: AsyncM3ter) -> None:
        data_export = await async_client.data_exports.create_adhoc(
            source_type="USAGE",
            account_ids=["string"],
            aggregations=[
                {
                    "field_code": "x",
                    "field_type": "DIMENSION",
                    "function": "SUM",
                    "meter_id": "x",
                }
            ],
            dimension_filters=[
                {
                    "field_code": "x",
                    "meter_id": "x",
                    "values": ["string"],
                }
            ],
            end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            groups=[{"group_type": "ACCOUNT"}],
            meter_ids=["string"],
            version=0,
        )
        assert_matches_type(AdHocResponse, data_export, path=["response"])

    @parametrize
    async def test_raw_response_create_adhoc_overload_2(self, async_client: AsyncM3ter) -> None:
        response = await async_client.data_exports.with_raw_response.create_adhoc(
            source_type="USAGE",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        data_export = await response.parse()
        assert_matches_type(AdHocResponse, data_export, path=["response"])

    @parametrize
    async def test_streaming_response_create_adhoc_overload_2(self, async_client: AsyncM3ter) -> None:
        async with async_client.data_exports.with_streaming_response.create_adhoc(
            source_type="USAGE",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            data_export = await response.parse()
            assert_matches_type(AdHocResponse, data_export, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Annotated, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo

__all__ = ["ContractListParams"]


class ContractListParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    account_id: Annotated[Optional[str], PropertyInfo(alias="accountId")]

    codes: SequenceNotStr[str]
    """
    An optional parameter to retrieve specific Contracts based on their short codes.
    """

    ids: SequenceNotStr[str]
    """
    An optional parameter to filter the list based on specific Contract unique
    identifiers (UUIDs).
    """

    next_token: Annotated[str, PropertyInfo(alias="nextToken")]
    """The `nextToken` for multi-page retrievals.

    It is used to fetch the next page of Contracts in a paginated list.
    """

    page_size: Annotated[int, PropertyInfo(alias="pageSize")]
    """Specifies the maximum number of Contracts to retrieve per page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["StatementJobListParams"]


class StatementJobListParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    active: str
    """Boolean filter on whether to only retrieve active \\**(i.e.

    not completed/cancelled)\\** StatementJobs.

    - TRUE - only active StatementJobs retrieved.
    - FALSE - all StatementJobs retrieved.
    """

    bill_id: Annotated[str, PropertyInfo(alias="billId")]
    """Filter Statement Jobs by billId"""

    next_token: Annotated[str, PropertyInfo(alias="nextToken")]
    """The `nextToken` for multi-page retrievals.

    It is used to fetch the next page of StatementJobs in a paginated list.
    """

    page_size: Annotated[int, PropertyInfo(alias="pageSize")]
    """Specifies the maximum number of StatementJobs to retrieve per page."""

    status: str
    """Filter using the StatementJobs status. Possible values:

    - `PENDING`
    - `RUNNING`
    - `COMPLETE`
    - `CANCELLED`
    - `FAILED`
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["LookupTableRevisionDataJobListParams"]


class LookupTableRevisionDataJobListParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    lookup_table_id: Required[Annotated[str, PropertyInfo(alias="lookupTableId")]]

    next_token: Annotated[str, PropertyInfo(alias="nextToken")]
    """The nextToken for multi page retrievals"""

    page_size: Annotated[int, PropertyInfo(alias="pageSize")]
    """The number of Lookup Table Revision Data Job Responses to retrieve per page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["ObjectURLResponse"]


class ObjectURLResponse(BaseModel):
    download_url: Optional[str] = FieldInfo(alias="downloadUrl", default=None)
    """The pre-signed download URL."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Iterable
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo

__all__ = ["StatementDefinitionCreateParams", "Dimension", "Measure"]


class StatementDefinitionCreateParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    aggregation_frequency: Required[
        Annotated[
            Literal["DAY", "WEEK", "MONTH", "QUARTER", "YEAR", "WHOLE_PERIOD"],
            PropertyInfo(alias="aggregationFrequency"),
        ]
    ]
    """This specifies how often the Statement should aggregate data."""

    dimensions: Iterable[Dimension]
    """
    An array of objects, each representing a Dimension data field from a Meter _(for
    Meters that have Dimensions setup)_.
    """

    generate_slim_statements: Annotated[bool, PropertyInfo(alias="generateSlimStatements")]

    include_price_per_unit: Annotated[bool, PropertyInfo(alias="includePricePerUnit")]
    """A Boolean indicating whether to include the price per unit in the Statement.

    - TRUE - includes the price per unit.
    - FALSE - excludes the price per unit.
    """

    measures: Iterable[Measure]
    """An array of objects, each representing a Measure data field from a Meter."""

    name: str
    """Descriptive name for the StatementDefinition providing context and information."""

    version: int
    """The version number of the entity:

    - **Create entity:** Not valid for initial insertion of new entity - _do not use
      for Create_. On initial Create, version is set at 1 and listed in the
      response.
    - **Update Entity:** On Update, version is required and must match the existing
      version because a check is performed to ensure sequential versioning is
      preserved. Version is incremented by 1 and listed in the response.
    """


class Dimension(TypedDict, total=False):
    """A Dimension belonging to a Meter."""

    filter: Required[SequenceNotStr[str]]
    """The value of a Dimension to use as a filter.

    Use "\\**" as a wildcard to filter on all Dimension values.
    """

    name: Required[str]
    """The name of the Dimension to target in the Meter."""

    attributes: SequenceNotStr[str]
    """The Dimension attribute to target."""

    meter_id: Annotated[str, PropertyInfo(alias="meterId")]
    """The unique identifier (UUID) of the Meter containing this Dimension."""


class Measure(TypedDict, total=False):
    aggregations: List[Literal["SUM", "MIN", "MAX", "COUNT", "LATEST", "MEAN", "UNIQUE", "CUSTOM_SQL"]]
    """A list of Aggregations to apply to the Measure."""

    meter_id: Annotated[str, PropertyInfo(alias="meterId")]
    """The unique identifier (UUID) of the Meter containing this Measure."""

    name: str
    """The name of a Measure data field \\**(or blank to indicate a wildcard, i.e.

    all fields)\\**. Default value is blank.
    """
